/**************************************************************************
 *   global.c                                                             *
 *                                                                        *
 *   Copyright (C) 1999 Chris Allegretta                                  *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 1, or (at your option)  *
 *   any later version.                                                   *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, write to the Free Software          *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
 *                                                                        *
 **************************************************************************/

#include <sys/stat.h>
#include "config.h"
#include "nano.h"
#include "proto.h"

/*
 * Global variables
 */
int center_x = 0, center_y = 0;	/* Center of screen */
WINDOW *edit;			/* The file portion of the editor  */
WINDOW *topwin;			/* Top line of screen */
WINDOW *bottomwin;		/* Bottom buffer */
int file;			/* Actual file pointer */
char *filename;			/* Name of the file */
int modified = 0;		/* Has file been modified? */
struct stat fileinfo;		/* Informatio about the file */
int editwinrows = 0;		/* How many rows long is the edit
				   window? */
int editwineob = 0;		/* Last Line in edit buffer
				   (0 - editwineob) */
filestruct *current;		/* Current buffer pointer */
int current_x = 0, current_y = 0;	/* Current position of X and Y in
					   the editor - relative to edit
					   window (0,0) */
filestruct *fileage = NULL;	/* Our file buffer */
filestruct *edittop = NULL;	/* Pointer to the top of the edit
				   buffer with respect to the
				   file struct */
filestruct *editbot = NULL;	/* Same for the bottom */
filestruct *filebot = NULL;	/* Last node in the file struct */
filestruct *cutbuffer = NULL;	/* A place to store cut text */
filestruct *cutbottom = NULL;	/* Pointer to end of cutbuffer */
int keep_cutbuffer = 0;		/* Clear out the cutbuffer? */

char answer[132] = "";		/* Answer str to many questions */
char last_search[132] = "";	/* Last string we searched for */
char last_replace[132] = "";	/* Last replacement string */
int totlines = 0;		/* Total number of lines in the file */
int totsize = 0;		/* Total number of bytes in the file */
int suspend = 0;		/* Can nano be suspended */
int case_sensitive = 0;		/* Are we doing case sensitive
				   searches */
int placewewant = 0;		/* The collum we'd like the cursor
				   to jump to when we go to the
				   next or previous line */

int statblank = 0;		/* Number of keystrokes left after
				   we call statubar() before we
				   actually blank the statusbar */
int mark_isset = 0;		/* Is the marker currently active */
int no_wrap = 0;		/* Is wrapping turned on? */
int autoindent = 0;		/* Autoindent turned on? */
int samelinewrap = 0;		/* Wrap to the beginning of the next line 
				   instead of starting a new one? */
int constupdate = 0;		/* Do we always show the cursor pos on statusbar? */
int no_help = 0;		/* No help at the bottom of the screen */
int help_mode = 0;		/* Are we in help mode (^G) */
int temp_opt = 0;		/* Editing temp file (pico -t option) */
int view_mode = 0;		/* View (read-only, uh kinda) mode */
int fill = 0;			/* Fill - where to wrap lines, basically */
int use_mouse = 0;		/* Enable mouse events. */
int pico_msgs = 0;		/* Make the shortcut lists like more like pico */

char *hblank;			/* A horizontal blank line */
char *alt_speller;		/* Alternative spell command */
char *help_text;		/* The text in the help window */
char *help_text_init =		/* Initial message, not including shortcuts */
" nano help text\n\n "
"The nano editor is designed to emulate the functionality and "
"ease-of-use of the UW Pico text editor.  There are four main "
"sections of the editor: The top line shows the program "
"version, the current filename being edited, and whether "
"or not the file has been modified.  Next is the main editor "
"window showing the file being edited.  The status line is "
"the third line from the bottom and shows important messages. "
"The bottom two lines show the most commonly used shortcuts "
"in the editor.\n\n "
"The notation for shortcuts is as follows: Control-key "
"sequences are notated with a caret (^) symbol.  Alt-key "
"sequences are notated with an at (@) symbol.  The following "
"keystrokes are available in the main editor window. "
"Optional keys are shown in parentheses:\n\n";

/* More stuff for the marker select */

filestruct *mark_beginbuf;	/* the begin marker buffer */
int mark_beginx;		/* X value in the string to start */
int marked_cut;			/* Is the cutbuffer from a mark */

shortcut main_list[MAIN_LIST_LEN];
shortcut whereis_list[WHEREIS_LIST_LEN];
shortcut replace_list[REPLACE_LIST_LEN];
shortcut goto_list[GOTO_LIST_LEN];
shortcut writefile_list[WRITEFILE_LIST_LEN];
shortcut help_list[HELP_LIST_LEN];

/* Initialize a struct *without* our lovely braces =( */
void sc_init_one(shortcut * s, int key, char *desc, char *help, int alt,
		 int misc1, int misc2, int view, int (*func) (void))
{
    s->val = key;
    s->desc = desc;
    s->help = help;
    s->altval = alt;
    s->misc1 = misc1;
    s->misc2 = misc2;
    s->viewok = view;
    s->func = func;
}

void shortcut_init(void)
{
    if (pico_msgs)
	sc_init_one(&main_list[0], NANO_HELP_KEY, "Get Help",
		    "Invoke the help menu", 0, 0, 0, VIEW, do_help);
    else
	sc_init_one(&main_list[0], NANO_WRITEOUT_KEY, "WriteOut",
		    "Write the current file to disk",
		    0, NANO_WRITEOUT_FKEY, 0, NOVIEW, do_writeout_void);


    sc_init_one(&main_list[1], NANO_EXIT_KEY, "Exit", "Exit from nano",
		0, NANO_EXIT_FKEY, 0, VIEW, do_exit);

    if (pico_msgs)
	sc_init_one(&main_list[2], NANO_WRITEOUT_KEY, "WriteOut",
		    "Write the current file to disk",
		    0, NANO_WRITEOUT_FKEY, 0, NOVIEW, do_writeout_void);
    else
	sc_init_one(&main_list[2], NANO_GOTO_KEY, "Goto Line",
		    "Goto a specific line number",
		  NANO_ALT_G, NANO_GOTO_FKEY, 0, VIEW, do_gotoline_void);

    if (pico_msgs)
	sc_init_one(&main_list[3], NANO_JUSTIFY_KEY, "Justify",
		    "Justify the current paragraph", 0, 0, 0, NOVIEW,
		    do_justify);
    else
	sc_init_one(&main_list[3], NANO_REPLACE_KEY, "Replace",
		    "Replace text within the editor",
		    NANO_ALT_R, NANO_REPLACE_FKEY, 0, NOVIEW, do_replace);

    sc_init_one(&main_list[4], NANO_INSERTFILE_KEY, "Read File",
		"Insert another file into the current one",
		0, NANO_INSERTFILE_FKEY, 0, NOVIEW, do_insertfile);

    sc_init_one(&main_list[5], NANO_WHEREIS_KEY, "Where Is",
		"Search for text within the editor",
		0, NANO_WHEREIS_FKEY, 0, VIEW, do_search);

    sc_init_one(&main_list[6], NANO_PREVPAGE_KEY, "Prev Page",
		"Move to the previous screen",
		0, NANO_PREVPAGE_FKEY, KEY_PPAGE, VIEW, page_up);

    sc_init_one(&main_list[7], NANO_NEXTPAGE_KEY, "Next Page",
		"Move to the next screen",
		0, NANO_NEXTPAGE_FKEY, KEY_NPAGE, VIEW, page_down);

    sc_init_one(&main_list[8], NANO_CUT_KEY, "Cut Text",
		"Cut the current line and store it in the cutbuffer",
		0, NANO_CUT_FKEY, 0, NOVIEW, do_cut_text);

    sc_init_one(&main_list[9], NANO_UNCUT_KEY, "UnCut Txt",
		"Uncut from the cutbuffer into the current line",
		0, NANO_UNCUT_FKEY, 0, NOVIEW, do_uncut_text);

    sc_init_one(&main_list[10], NANO_CURSORPOS_KEY, "Cur Pos",
		"Show the posititon of the cursor",
		0, NANO_CURSORPOS_FKEY, 0, VIEW, do_cursorpos);

    sc_init_one(&main_list[11], NANO_SPELL_KEY, "To Spell",
		"Invoke the spell checker (if available)",
		0, NANO_SPELL_FKEY, 0, NOVIEW, do_spell);


    sc_init_one(&main_list[12], NANO_UP_KEY, "Up",
		"Move up one line", 0, KEY_UP, 0, VIEW, do_up);

    sc_init_one(&main_list[13], NANO_DOWN_KEY, "Down",
		"Move down one line", 0, KEY_DOWN, 0, VIEW, do_down);

    sc_init_one(&main_list[14], NANO_FORWARD_KEY, "Forward",
		"Move forward one character", 0, KEY_RIGHT, 0, VIEW,
		do_right);

    sc_init_one(&main_list[15], NANO_BACK_KEY, "Back",
		"Move back one character", 0, KEY_LEFT, 0, VIEW, do_left);

    sc_init_one(&main_list[16], NANO_HOME_KEY, "Home",
		"Move to the beginning of the current line", 0, KEY_HOME,
		362, VIEW, do_home);

    sc_init_one(&main_list[17], NANO_END_KEY, "End",
		"Move to the end of the current line", 0, KEY_END, 385,
		VIEW, do_end);

    sc_init_one(&main_list[18], NANO_REFRESH_KEY, "Refresh",
		"Refresh (redraw) the current screen", 0, 0, 0, VIEW,
		total_refresh);

    sc_init_one(&main_list[19], NANO_MARK_KEY, "Mark Text",
		"Mark text at the current cursor location", 0, 0, 0,
		NOVIEW, do_mark);

    sc_init_one(&main_list[20], NANO_DELETE_KEY, "Delete",
		"Delete the character under the cursor", 0, KEY_DC,
		NANO_CONTROL_D, NOVIEW, do_delete);

    sc_init_one(&main_list[21], NANO_BACKSPACE_KEY, "Backspace",
		"Delete the character to the left of the cursor", 0,
		KEY_BACKSPACE, NANO_CONTROL_H, NOVIEW, do_backspace);

    sc_init_one(&main_list[22], NANO_TAB_KEY, "Tab",
		"Insert a tab character", 0, 0, 0, NOVIEW, do_tab);

    sc_init_one(&main_list[23], NANO_JUSTIFY_KEY, "Justify",
		"Justify the current paragraph", 0, 0, 0, NOVIEW,
		do_justify);

    sc_init_one(&main_list[24], NANO_SUSPEND_KEY, "Suspend",
		"Suspend nano if suspend is enabled", 0, 0, 407, VIEW,
		do_suspend);

    sc_init_one(&main_list[25], NANO_ENTER_KEY, "Enter",
		"Insert a carriage return at the cursor position", 0,
		KEY_ENTER, NANO_CONTROL_M, NOVIEW, do_enter_void);

    sc_init_one(&main_list[26], NANO_HELP_KEY, "Get Help",
		"Invoke the help menu", 0, 0, 0, VIEW, do_help);


    sc_init_one(&whereis_list[0], NANO_FIRSTLINE_KEY, "First Line",
		"Go to the first line of the file",
		0, 0, 0, VIEW, do_first_line);

    sc_init_one(&whereis_list[1], NANO_LASTLINE_KEY, "Last Line",
		"Go to the last line of the file",
		0, 0, 0, VIEW, do_last_line);

    sc_init_one(&whereis_list[2], NANO_CASE_KEY, "Case Sens",
		"Make the current search or replace case (in)sensitive",
		0, 0, 0, VIEW, 0);

    sc_init_one(&whereis_list[3], NANO_CANCEL_KEY, "Cancel",
		"Cancel the current function", 0, 0, 0, VIEW, 0);


    sc_init_one(&replace_list[0], NANO_FIRSTLINE_KEY, "First Line",
		"Go to the first line of the file",
		0, 0, 0, VIEW, do_first_line);

    sc_init_one(&replace_list[1], NANO_LASTLINE_KEY, "Last Line",
		"Go to the last line of the file",
		0, 0, 0, VIEW, do_last_line);

    sc_init_one(&replace_list[2], NANO_CASE_KEY, "Case Sens",
		"Make the current search or replace case (in)sensitive",
		0, 0, 0, VIEW, 0);

    sc_init_one(&replace_list[3], NANO_CANCEL_KEY, "Cancel",
		"Cancel the current function", 0, 0, 0, VIEW, 0);


    sc_init_one(&goto_list[0], NANO_FIRSTLINE_KEY, "First Line",
		"Go to the first line of the file",
		0, 0, 0, VIEW, &do_first_line);

    sc_init_one(&goto_list[1], NANO_LASTLINE_KEY, "Last Line",
		"Go to the last line of the file",
		0, 0, 0, VIEW, &do_last_line);

    sc_init_one(&goto_list[2], NANO_CANCEL_KEY, "Cancel",
		"Cancel the current function", 0, 0, 0, VIEW, 0);


    sc_init_one(&help_list[0], NANO_PREVPAGE_KEY, "Prev Page",
		"Move to the previous screen",
		0, NANO_PREVPAGE_FKEY, KEY_PPAGE, VIEW, page_up);

    sc_init_one(&help_list[1], NANO_NEXTPAGE_KEY, "Next Page",
		"Move to the next screen",
		0, NANO_NEXTPAGE_FKEY, KEY_NPAGE, VIEW, page_down);

    sc_init_one(&help_list[2], NANO_EXIT_KEY, "Exit", "Exit from nano",
		0, NANO_EXIT_FKEY, 0, VIEW, do_exit);


    sc_init_one(&writefile_list[0], NANO_CANCEL_KEY, "Cancel",
		"Cancel the current function", 0, 0, 0, VIEW, 0);

}
